from isfilled.models import Fills
from isfilled.util import import_string

class Filled(object):
    def __init__(self, instance=None):
        self.instance = instance

    def default_fields(self):
        if self.instance and self.instance.model:
            res = [k.name for k in self.instance.model_model()._meta.get_fields()]
        elif self.instance.fill:
            res = self.form().fields.keys()
        else:
            raise Exception("Something is wrong")
        return res

    def fields(self, keys, exclude=[]):
        return list(set(keys) - set(exclude))

    def is_filled(self):
        return True

    def is_field_filled(self, instance, field):
        # not all form fields are model fields; ignore
        if not hasattr(instance, field):
            return True
        return getattr(instance, field)

    def verify(self, instance, fields):
        return all(self.is_field_filled(instance, k) for k in fields)

class FillsMixin(object):
    """ Fills superpowers for Models """

    def registered_model_fills(self):
        return Fills.objects.filter(model=self._meta.label)

    def as_fills(self, fills):
        return [import_string(fill.fill) if fill.fill else None for fill in fills]

    def check_fills(self, fills=[]):
        registered_model_fills = fills or self.registered_model_fills()
        registered_fills = self.as_fills(registered_model_fills)
        def _fields(fill, instance):
            return fill.fields(keys=instance.fields or fill.default_fields(),
                        exclude=instance.exclude or [],)
        def _verify(fill, instance):
            fill_instance = fill(instance=instance) if fill else Filled(instance=instance)
            flds = _fields(fill_instance, instance)
            return fill_instance.verify(self, flds)

        ctxs = {fill: _verify(fill, ins) for fill, ins in 
                    zip(registered_fills, registered_model_fills)}
        return (all(ctxs.values()), ctxs)

    def check_fill(self, fill):
        self.check_fills([fill])




import os, importlib

def import_string(dotted_path):
    try:
        module_path, class_name = dotted_path.rsplit('.', 1)
    except (ValueError, AttributeError) as e:
        module_path = dotted_path
    try:
        return importlib.import_module(dotted_path)
    except ImportError, e:
        module = importlib.import_module(module_path)
        return getattr(module, class_name)



# -*- coding: utf-8 -*-
from django.apps import apps
from django.core.cache import cache
from django.conf import settings
from django.db import models
from django.db.models import Count

from isfilled.util import import_string

class Fills(models.Model):
    created = models.DateTimeField(auto_now_add=True)
    name = models.CharField(max_length=255)
    fill = models.CharField(max_length=255, null=True, blank=True, db_index=True)
    model = models.CharField(max_length=255, null=True, blank=True, db_index=True)
    fields = models.CharField(max_length=255, null=True, blank=True)
    exclude = models.CharField(max_length=255, null=True, blank=True)

    def fill_model(self):
        fills = import_string(self.fill)
        return fills.form.Meta.model

    def model_model(self):
        return apps.get_model(self.model)

    def save(self, *args, **kwargs):
        if self.fill and not self.model:
            model = self.fill_model()
            self.model = model._meta.label
        super(Fills, self).save(*args, **kwargs)

    def __unicode__(self):
        return self.name

    class Meta:
        unique_together = (('name', 'fill', 'model'),)




# -*- coding: utf-8 -*-
from django.apps import apps
from django.conf import settings
from django.contrib.contenttypes.models import ContentType
from django.core.exceptions import ValidationError, ObjectDoesNotExist
from django.db import models
from django.test import TestCase, TransactionTestCase, override_settings
from django.test.client import Client
from django.contrib.auth.models import Group, User
from django import forms

from isfilled.models import Fills
from isfilled.fills import FillsMixin, Filled

import os, logging, datetime
logging.basicConfig(level=logging.DEBUG)

class BaseSuite(TransactionTestCase):
    pass

class Employee(models.Model, FillsMixin):
    name = models.CharField(max_length=255)
    expert_in = models.CharField(max_length=255, blank=True, null=True)
    starting_date = models.DateField(blank=True, null=True)
    editor_of_choice = models.CharField(max_length=255, blank=True, null=True)

class EmployeeFillsForm(forms.ModelForm):
    class Meta:
        model = Employee
        exclude = []

class EmployeeFills(Filled):
    form = EmployeeFillsForm

class FillsTest(BaseSuite):
    def test_form_fills(self):
        e = Employee.objects.create(name="Jared")

        f = Fills.objects.create(
                name="employee-hr",
                fill="test.test_fills.EmployeeFills",)

        state, ctx = e.check_fills()
        self.assertEqual(state, False)

        e.expert_in = "iOS"
        e.starting_date = datetime.date.today()
        e.editor_of_choice = 'vim'
        e.save()

        state, ctx = e.check_fills()
        self.assertEqual(state, True)

    def test_model_fills(self):
        e = Employee.objects.create(name="Jared")

        f = Fills.objects.create(
                name="employee",
                model="test.Employee",)

        state, ctx = e.check_fills()
        self.assertEqual(state, False)

        e.expert_in = "Android C++ C#"
        e.starting_date = datetime.date.today()
        e.editor_of_choice = 'vim'
        e.save()

        state, ctx = e.check_fills()
        self.assertEqual(state, True)




#!/usr/bin/env python
from setuptools import setup, find_packages, Command
from setuptools.command.test import test

import os, sys, subprocess

class TestCommand(Command):
    user_options = []

    def initialize_options(self):
        pass

    def finalize_options(self):
        pass

    def run(self):
        raise SystemExit(
            subprocess.call([sys.executable,
                             'app_test_runner.py',
                             'test']))

install_requires = ['']

setup(
    name = "isfilled",
    version = "0.1",
    description = "Filledness for Models and ModelForms",
    url = "http://github.com/futurice/isfilled",
    author = "Jussi Vaihia",
    author_email = "jussi.vaihia@futurice.com",
    packages = ["isfilled"],
    include_package_data = True,
    keywords = 'django form filled condition',
    license = 'BSD',
    install_requires = install_requires,
    cmdclass = {
        'test': TestCommand,
    },
)
